"""Tests decoder registration functionality."""

import os

import kordesii
from kordesii.decoder import Decoder


def test_register_decoder_directory(monkeypatch, Sample_decoder):
    # Monkey patch decoders registration so previous test runs don't muck with this.
    monkeypatch.setattr("kordesii.registry._sources", {})

    decoder_dir = Sample_decoder.dirname

    # Test registration
    assert not list(kordesii.iter_decoders("Sample"))
    kordesii.register_decoder_directory(decoder_dir)
    decoders = list(kordesii.iter_decoders("Sample"))
    assert len(decoders) == 1

    # Test it was registered properly
    decoder = decoders[0]
    assert decoder.name == "Sample"

    # Test we can also pull by source name.
    decoders = list(kordesii.iter_decoders(source=decoder_dir))
    assert len(decoders) == 1
    decoders = list(kordesii.iter_decoders(decoder_dir + ":"))
    assert len(decoders) == 1


def test_register_decoder_directory2(monkeypatch, Sample_decoder):
    # Monkey patch decoders registration so previous test runs don't muck with this.
    monkeypatch.setattr("kordesii.registry._sources", {})

    decoder_dir = Sample_decoder.dirname

    # Test registration
    assert not list(kordesii.iter_decoders("Sample"))
    kordesii.register_decoder_directory(decoder_dir, source_name="ACME")
    decoders = list(kordesii.iter_decoders("Sample"))
    assert len(decoders) == 1

    # Test it was registered properly
    decoder = decoders[0]
    assert decoder.name == "Sample"
    assert decoder.source.name == "ACME"
    assert decoder.source.path == decoder_dir

    # Test we can also pull by source name.
    decoders = list(kordesii.iter_decoders(source="ACME"))
    assert len(decoders) == 1
    decoders = list(kordesii.iter_decoders("ACME:"))
    assert len(decoders) == 1


def test_iter_decoders(monkeypatch, Sample_decoder):
    monkeypatch.setattr("kordesii.registry._sources", {})

    source = os.path.abspath(Sample_decoder.dirname)
    kordesii.register_decoder_directory(source)

    decoders = list(kordesii.iter_decoders("Sample"))
    assert len(decoders) == 1

    decoder = decoders[0]
    assert isinstance(decoder, Decoder)
    assert decoder.name == "Sample"

    decoders = list(kordesii.iter_decoders(source=source))
    assert len(decoders) == 1

    decoder = decoders[0]
    assert isinstance(decoder, Decoder)
    assert decoder.name == "Sample"

    assert list(kordesii.iter_decoders(name="bogus")) == []
    assert list(kordesii.iter_decoders(source="bogus")) == []



import io
import logging

import pytest

import kordesii
from kordesii.tools.server import create_app


@pytest.fixture(scope="module")
def client():
    app = create_app()
    with app.test_client() as client:
        yield client


@pytest.fixture(scope="module")
def decoders():
    return list(kordesii.iter_decoders())


@pytest.fixture
def sample_file_pointer(strings_exe):
    with io.open(str(strings_exe), "rb") as f:
        yield f


@pytest.fixture
def expected_results():
    return {
        u"debug": [
            u"[+] (kordesii.decoders.sample): Found XOR encrypt function at: 0x401000",
            u"[+] (kordesii.core): IDA return code = 0",
        ],
        u"error": [],
        u"output_text": (
            u"----Decoded Strings----\n\n"
            u"Hello World!\n"
            u"Test string with key 0x02\n"
            u"The quick brown fox jumps over the lazy dog.\n"
            u"Oak is strong and also gives shade.\n"
            u"Acid burns holes in wool cloth.\n"
            u"Cats and dogs each hate the other.\n"
            u"Open the crate but don't break the glass.\n"
            u"There the flood mark is ten inches.\n"
            u"1234567890\n"
            u"CreateProcessA\n"
            u"StrCat\n"
            u"ASP.NET\n"
            u"kdjsfjf0j24r0j240r2j09j222\n"
            u"32897412389471982470\n"
            u"The past will look brighter tomorrow.\n"
            u"Cars and busses stalled in sand drifts.\n"
            u"The jacket hung on the back of the wide chair.\n"
            u"32908741328907498134712304814879837483274809123748913251236598123056231895712\n\n"
            u"----Debug----\n\n"
            u"[+] (kordesii.decoders.sample): Found XOR encrypt function at: 0x401000\n"
            u"[+] (kordesii.core): IDA return code = 0\n"
        ),
        u"strings": [
            u"Hello World!",
            u"Test string with key 0x02",
            u"The quick brown fox jumps over the lazy dog.",
            u"Oak is strong and also gives shade.",
            u"Acid burns holes in wool cloth.",
            u"Cats and dogs each hate the other.",
            u"Open the crate but don't break the glass.",
            u"There the flood mark is ten inches.",
            u"1234567890",
            u"CreateProcessA",
            u"StrCat",
            u"ASP.NET",
            u"kdjsfjf0j24r0j240r2j09j222",
            u"32897412389471982470",
            u"The past will look brighter tomorrow.",
            u"Cars and busses stalled in sand drifts.",
            u"The jacket hung on the back of the wide chair.",
            u"32908741328907498134712304814879837483274809123748913251236598123056231895712",
        ],
    }


@pytest.fixture(scope="module")
def make_test_buffer():
    # The client closes the file-object when it's completed its request
    # so we need to be able to generate a new one when needed
    def _make_test_buffer():
        return io.BytesIO(b"This is a test file!\n")

    return _make_test_buffer


def test_homepage(client):
    """Test the homepage is accessible"""
    rv = client.get("/")
    assert rv.status_code == 200
    assert b"DC3-kordesii Service" in rv.data


def test_menu(client):
    """Test menu items can be added"""
    # Menu links can be created via adding to the config
    client.application.config["MENU_LINKS"].append({"name": "Example", "url": "http://example.com"})
    rv = client.get("/")
    assert b'<li><a href="/">Home</a></li>' in rv.data
    assert b'<li><a href="http://example.com">Example</a></li>' in rv.data


def test_log_endpoint(client):
    from kordesii import logutil

    rv = client.get("/logs")

    assert {"error": ["No 'kordesii_server' handler defined on root logger."]} == rv.json
    assert rv.status_code == 500

    list_handler = logutil.ListHandler()
    list_handler.name = "kordesii_server"
    logging.root.addHandler(list_handler)

    rv = client.get("/logs")
    assert rv.status_code == 200
    assert isinstance(rv.json, dict)
    assert "logs" in rv.json
    logging.root.removeHandler(list_handler)

    client.application.config["DISABLE_LOGS_ENDPOINT"] = True
    rv = client.get("/logs")
    assert rv.status_code == 403
    assert {"error": ["Logs endpoint has been disabled by configuration"]} == rv.json


@pytest.mark.in_ida
def test_decoder_strings(client, expected_results, sample_file_pointer):
    """Test decoder strings output"""
    rv = client.post("/run_decoder", data={"decoder": "Sample", "input_file": (sample_file_pointer, "strings.exe")})
    # Print results for debugging purposes.
    print(rv)
    print(rv.json)
    assert rv.json["strings"] == expected_results["strings"]


def test_upload_options(client, decoders):
    """Test the upload page lists all decoders"""
    rv = client.get("/upload")

    option_str = '<option value="{name}">{name}</option>'

    for decoder in decoders:
        assert option_str.format(name=decoder.name).encode() in rv.data


def test_decoders(client, decoders):
    """Test the HTML decoders page lists the decoders"""
    import flask

    rv = client.get("/decoders")

    example_row = """\
            <tr>
                
                    <td>{name}</td>
                
                    <td>{source}</td>
                
                    <td>{description}</td>
                
            </tr>"""

    for decoder in decoders:
        # Each string must be escaped, this is esp. for descriptions
        escaped_info = {
            "name": flask.escape(decoder.name),
            "source": flask.escape(decoder.source.name),
            "description": flask.escape(decoder.description),
        }
        row = example_row.format(**escaped_info).encode()
        assert row in rv.data


def test_decoders_json(client, decoders):
    """Test the JSON decoders response lists all decoders"""
    rv = client.get("/decoders", headers={"Accept": "application/json"})

    assert rv.content_type == "application/json"
    assert isinstance(rv.json, dict)
    assert "decoders" in rv.json

    decoders_json = rv.json["decoders"]

    assert len(decoders_json) == len(decoders)

    for decoder in decoders:
        decoder_info = {"name": decoder.name, "source": decoder.source.name, "description": decoder.description}
        assert decoder_info in decoders_json


def test_descriptions(client, decoders):
    """Test the legacy descriptions endpoint lists all decoders"""
    rv = client.get("/descriptions")
    assert "decoders" in rv.json
    decoders_descriptions = rv.json["decoders"]
    assert decoders_descriptions
    assert isinstance(decoders_descriptions, list)
    assert len(decoders_descriptions) == len(decoders)

    for idx, decoder in enumerate(decoders):
        assert decoders_descriptions[idx] == decoder.name



"""
Tests utility functions.
"""

import pytest


@pytest.mark.in_ida
def test_iter_functions():
    import idc
    from kordesii.utils import utils

    assert list(utils.iter_exports()) == [(0x4014e0, 'start')]
    assert list(utils.iter_imports()) == [
        (0x40a000, 'GetCommandLineA', 'KERNEL32'),
        (0x40a004, 'EnterCriticalSection', 'KERNEL32'),
        (0x40a008, 'LeaveCriticalSection', 'KERNEL32'),
        (0x40a00c, 'TerminateProcess', 'KERNEL32'),
        (0x40a010, 'GetCurrentProcess', 'KERNEL32'),
        (0x40a014, 'UnhandledExceptionFilter', 'KERNEL32'),
        (0x40a018, 'SetUnhandledExceptionFilter', 'KERNEL32'),
        (0x40a01c, 'IsDebuggerPresent', 'KERNEL32'),
        (0x40a020, 'GetModuleHandleW', 'KERNEL32'),
        (0x40a024, 'Sleep', 'KERNEL32'),
        (0x40a028, 'GetProcAddress', 'KERNEL32'),
        (0x40a02c, 'ExitProcess', 'KERNEL32'),
        (0x40a030, 'WriteFile', 'KERNEL32'),
        (0x40a034, 'GetStdHandle', 'KERNEL32'),
        (0x40a038, 'GetModuleFileNameA', 'KERNEL32'),
        (0x40a03c, 'FreeEnvironmentStringsA', 'KERNEL32'),
        (0x40a040, 'GetEnvironmentStrings', 'KERNEL32'),
        (0x40a044, 'FreeEnvironmentStringsW', 'KERNEL32'),
        (0x40a048, 'WideCharToMultiByte', 'KERNEL32'),
        (0x40a04c, 'GetLastError', 'KERNEL32'),
        (0x40a050, 'GetEnvironmentStringsW', 'KERNEL32'),
        (0x40a054, 'SetHandleCount', 'KERNEL32'),
        (0x40a058, 'GetFileType', 'KERNEL32'),
        (0x40a05c, 'GetStartupInfoA', 'KERNEL32'),
        (0x40a060, 'DeleteCriticalSection', 'KERNEL32'),
        (0x40a064, 'TlsGetValue', 'KERNEL32'),
        (0x40a068, 'TlsAlloc', 'KERNEL32'),
        (0x40a06c, 'TlsSetValue', 'KERNEL32'),
        (0x40a070, 'TlsFree', 'KERNEL32'),
        (0x40a074, 'InterlockedIncrement', 'KERNEL32'),
        (0x40a078, 'SetLastError', 'KERNEL32'),
        (0x40a07c, 'GetCurrentThreadId', 'KERNEL32'),
        (0x40a080, 'InterlockedDecrement', 'KERNEL32'),
        (0x40a084, 'HeapCreate', 'KERNEL32'),
        (0x40a088, 'VirtualFree', 'KERNEL32'),
        (0x40a08c, 'HeapFree', 'KERNEL32'),
        (0x40a090, 'QueryPerformanceCounter', 'KERNEL32'),
        (0x40a094, 'GetTickCount', 'KERNEL32'),
        (0x40a098, 'GetCurrentProcessId', 'KERNEL32'),
        (0x40a09c, 'GetSystemTimeAsFileTime', 'KERNEL32'),
        (0x40a0a0, 'GetCPInfo', 'KERNEL32'),
        (0x40a0a4, 'GetACP', 'KERNEL32'),
        (0x40a0a8, 'GetOEMCP', 'KERNEL32'),
        (0x40a0ac, 'IsValidCodePage', 'KERNEL32'),
        (0x409b0e, 'RtlUnwind', 'KERNEL32'),
        (0x40a0b0, 'RtlUnwind', 'KERNEL32'),
        (0x40a0b4, 'LoadLibraryA', 'KERNEL32'),
        (0x40a0b8, 'InitializeCriticalSectionAndSpinCount', 'KERNEL32'),
        (0x40a0bc, 'HeapAlloc', 'KERNEL32'),
        (0x40a0c0, 'VirtualAlloc', 'KERNEL32'),
        (0x40a0c4, 'HeapReAlloc', 'KERNEL32'),
        (0x40a0c8, 'GetConsoleCP', 'KERNEL32'),
        (0x40a0cc, 'GetConsoleMode', 'KERNEL32'),
        (0x40a0d0, 'FlushFileBuffers', 'KERNEL32'),
        (0x40a0d4, 'LCMapStringA', 'KERNEL32'),
        (0x40a0d8, 'MultiByteToWideChar', 'KERNEL32'),
        (0x40a0dc, 'LCMapStringW', 'KERNEL32'),
        (0x40a0e0, 'GetStringTypeA', 'KERNEL32'),
        (0x40a0e4, 'GetStringTypeW', 'KERNEL32'),
        (0x40a0e8, 'GetLocaleInfoA', 'KERNEL32'),
        (0x40a0ec, 'SetFilePointer', 'KERNEL32'),
        (0x40a0f0, 'HeapSize', 'KERNEL32'),
        (0x40a0f4, 'CloseHandle', 'KERNEL32'),
        (0x40a0f8, 'WriteConsoleA', 'KERNEL32'),
        (0x40a0fc, 'GetConsoleOutputCP', 'KERNEL32'),
        (0x40a100, 'WriteConsoleW', 'KERNEL32'),
        (0x40a104, 'SetStdHandle', 'KERNEL32'),
        (0x40a108, 'CreateFileA', 'KERNEL32')
    ]
    assert list(utils.iter_functions('GetProcAddress')) == [(0x40a028, 'GetProcAddress')]
    assert list(utils.iter_functions('memcpy')) == [(0x405c00, '_memcpy'), (0x408d50, '_memcpy_0')]
    assert list(utils.iter_functions('_memcpy')) == [(0x405c00, '_memcpy'), (0x408d50, '_memcpy_0')]
    assert list(utils.iter_functions('_memcpy_0')) == [(0x408d50, '_memcpy_0')]

    # There are no dynamically resolved functions in this sample, so we are just going to fake
    # it by naming a piece of data as if it was a function.
    assert list(utils.iter_dynamic_functions()) == []
    idc.SetType(0x40cdfc, "BOOL __stdcall WriteFile(HANDLE hFile, LPCVOID lpBuffer, DWORD nNumberOfBytesToWrite, LPDWORD lpNumberOfBytesWritten, LPOVERLAPPED lpOverlapped)")
    assert list(utils.iter_dynamic_functions()) == [(0x40cdfc, 'dword_40CDFC')]


@pytest.mark.in_ida
def test_iter_callers():
    """Tests utilities to pull callers."""
    from kordesii import utils

    assert list(utils.iter_calls_to(0x00401000)) == [
        0x40103a, 0x401049, 0x401058, 0x401067, 0x401076, 0x401085, 0x401094,
        0x4010a3, 0x4010b2, 0x4010c1, 0x4010d0, 0x4010df, 0x4010ee, 0x4010fd,
        0x40110c, 0x40111b, 0x40112a, 0x401139,
    ]
    callers = list(utils.iter_callers(0x00401000))
    assert len(callers) == 1
    caller = callers[0]
    assert caller.start_ea == 0x401030



from kordesii.decoder import Decoder


def test_metadata_extraction(Sample_decoder):
    decoder = Decoder(str(Sample_decoder))

    assert decoder.name == "Sample"
    assert decoder.author == "DC3"
    assert decoder.description == "Sample decoder"
    assert decoder.source.name == Sample_decoder.dirname
    assert decoder.source.path == Sample_decoder.dirname



"""
Tests the IDA proxy functionality.
"""

import pytest

import kordesii


pytestmark = pytest.mark.xfail(reason="IDA Proxy is deprecated.")


def test_basic(strings_exe):
    """Tests some basic functionality and stability."""
    strings_exe = str(strings_exe)

    with kordesii.IDA(strings_exe):
        import idc
        from kordesii.utils import utils
        from kordesii.utils import ida_re

        assert idc.get_input_file_path() == strings_exe
        assert idc.print_insn_mnem(0x00401525) == 'mov'
        assert utils.get_function_addr('GetProcAddress') == 0x40a028
        assert utils.get_string(0x0040C000) == b'Idmmn!Vnsme '

        regex = ida_re.Pattern(b'Idmmn!Vnsme')
        match = regex.search()
        assert match
        assert match.start() == 0x0040C000
        assert match.group() == b'Idmmn!Vnsme'

        # Ensure we can only start one at a time.
        with pytest.raises(ValueError):
            kordesii.IDA(r'C:\dummy.exe').start()
        pass

        # Test that we can also use submodules (the utils in utils)
        from kordesii import utils
        assert utils.get_function_addr('GetProcAddress') == 0x40a028
        assert utils.get_string(0x0040C000) == b'Idmmn!Vnsme '

        # Test that importing a class doesn't cause things to explode.
        try:
            from ida_gdl import BasicBlock
        except ImportError as e:
            # FIXME
            pytest.xfail(f"Known bug of IDA proxy: {e}")

        with pytest.raises(NotImplementedError) as exec_info:
            BasicBlock(1, 2, 3)
        assert str(exec_info.value) == "Initializing the class ida_gdl.BasicBlock is not supported."

    # Ensure we can't use modules after closing.
    with pytest.raises(AttributeError):
        idc.print_insn_mnem(0x00401525)

    # Now test that we can spin it up again.
    with kordesii.IDA(strings_exe):
        # import idc  # reimporting is not required.
        assert idc.get_input_file_path() == strings_exe

    # Now test manually starting and stopping.
    ida = kordesii.IDA(strings_exe)
    ida.start()
    import idc
    assert idc.get_input_file_path() == strings_exe
    ida.stop()

    # can't access imports outside
    with pytest.raises(AttributeError):
        idc.get_input_file_path()

    # now try starting the same instance again.
    ida.start()
    assert idc.get_input_file_path() == strings_exe
    ida.stop()

    # Try opening a file that is not actually an exe.
    # It should still work, just not be very helpful.
    with kordesii.IDA(__file__):
        assert idc.get_input_file_path() == __file__
        assert idc.print_insn_mnem(0x00401525) == ''


def ida_assert(value, expected):
    """
    Workaround to test values inside run_in_ida decorated functions.
    This is necessary because pytest's assert rewriting is messing with our ability
    to use assert in these functions.
    """
    if value != expected:
        raise AssertionError(f'{value} != {expected}')


@kordesii.run_in_ida
def trace_arguments(ea):
    """
    This is a function that would be almost impossible to do proxied due to
    the complexities of function_tracing.
    """
    from kordesii.utils import function_tracing

    emulator = function_tracing.Emulator()

    # Test that we can call other decorated functions.
    ida_assert(get_mnem(ea), 'mov')

    strings = []
    for context in emulator.iter_context_at(ea, depth=1):
        ida_assert(context.ip, ea)
        # mov     eax, [ebp+arg_0]
        strings.append(context.read_data(context.operands[1].value))

    return strings


@kordesii.run_in_ida
def get_mnem(ea):
    import idc
    return idc.print_insn_mnem(ea)


def test_run_in_ida(strings_exe):
    """Tests the run_in_ida decorator."""
    strings_exe = str(strings_exe)

    with kordesii.IDA(strings_exe):
        assert trace_arguments(0x00401003) == [
            b'Idmmn!Vnsme ',
            b'Vgqv"qvpkle"ukvj"ig{"2z20',
            b'Wkf#rvj`h#aqltm#el{#ivnsp#lufq#wkf#obyz#gld-',
            b'Keo$mw$wpvkjc$ej`$ehwk$cmraw$wle`a*',
            b'Dfla%gpwkv%mji`v%lk%rjji%fijqm+',
            b'Egru&ghb&biau&cgen&ngrc&rnc&irnct(',
            b'\\cv}3g{v3pargv3qfg3w|}4g3qavrx3g{v3t\x7fr``=',
            b'C\x7frer7c\x7fr7q{xxs7zve|7~d7cry7~yt\x7frd9',
            b'+()./,-"#*',
            b'`QFBWFsQL@FPPb',
            b'tSUdFS',
            b'\x01\x13\x10n\x0e\x05\x14',
            b'-",5 , v,tr4v,trv4t,v\x7f,ttt',
            b'@AKJDGBA@KJGDBJKAGDC',
            b'!\x1d\x10U\x05\x14\x06\x01U\x02\x1c\x19\x19U\x19\x1a\x1a\x1eU\x17\x07\x1c\x12\x1d\x01\x10\x07U\x01\x1a\x18\x1a\x07\x07\x1a\x02[',
            b'4\x16\x05\x04W\x16\x19\x13W\x15\x02\x04\x04\x12\x04W\x04\x03\x16\x1b\x1b\x12\x13W\x1e\x19W\x04\x16\x19\x13W\x13\x05\x1e\x11\x03\x04Y',
            b'.\x12\x1fZ\x10\x1b\x19\x11\x1f\x0eZ\x12\x0f\x14\x1dZ\x15\x14Z\x0e\x12\x1fZ\x18\x1b\x19\x11Z\x15\x1cZ\x0e\x12\x1fZ\r\x13\x1e\x1fZ\x19\x12\x1b\x13\x08T',
            b'LMFOGHKNLMGFOHKFGNLKHNMLOKGNKGHFGLHKGLMHKGOFNMLHKGFNLMJNMLIJFGNMLOJIMLNGFJHNM'
        ]

        # Also test that we can catch exceptions.
        with pytest.raises(TypeError, match="argument 2 of type 'ea_t'"):
            get_mnem('not an address')

    # Ensure we get an error, if we attempt to run the function outside of proxy instance.
    with pytest.raises(RuntimeError):
        trace_arguments(0x00401003)



"""
Tests serializer.
"""

import pytest

from kordesii import serialization


@serialization.serializable_class
class DummyObject(object):
    a = "a"
    b = "b"

    def __init__(self, c):
        self.c = c


expected_serialized = u"""\
entry_1: !!binary |
  c29tZSBkYXRh
entry_2: !ReadOnlyObject
  __yaml_name__: DummyObject
  a: a
  b: b
  c: c
entry_3:
- 1
- 2
- 3
"""


def test_Serializer(tmpdir):
    """Tests Serializer class"""
    yaml_file = tmpdir / "test.yml"
    dummy_object = DummyObject("c")

    with serialization.Serializer(str(yaml_file)) as serializer:
        assert not yaml_file.exists()

        serializer.set("entry_1", b"some data")
        serializer.set("entry_2", dummy_object)
        serializer.set("entry_3", [1, 2, 3])
        with pytest.raises(ValueError):
            # We can't reuse an already set attribute.
            serializer.set("entry_2", b"boop")

        assert yaml_file.exists()
        assert serializer.get("entry_1") == b"some data"
    # Must close manually because of hack needed to keep it global.
    serializer.close()

    # Test that our entry has been serialized.
    assert yaml_file.read_text("utf8") == expected_serialized


def test_deserialize(tmpdir):
    """Tests deserialization."""
    yaml_file = tmpdir / "test.yml"
    yaml_file.write_text(expected_serialized, "utf8")

    # Test deserialization by setting up a Serializer object as
    # well as using the deserialize() function.
    results = [
        serialization.deserialize(expected_serialized),
        # NOTE: this method doesn't work for now.
        # serialization.Serializer(str(yaml_file)).as_dict(),
    ]
    for result in results:
        assert result["entry_1"] == b"some data"
        assert result["entry_3"] == [1, 2, 3]
        dummy_object = result["entry_2"]
        assert isinstance(dummy_object, serialization.ROObject)
        assert dummy_object.a == "a"
        assert dummy_object.b == "b"
        assert dummy_object.c == "c"




import pytest


@pytest.mark.in_ida
def test_find_string_data():
    from kordesii.utils import find_string_data

    # General test of mixture of utf-8 and utf-16
    data = b"h\x00e\x00l\x00l\x00o\x00\x00\x00\x00\x00test\x00\x00\x00joe\x00\x00w\x00o\x00r\x00l\x00d"
    assert list(find_string_data(data)) == [
        (0, b"h\x00e\x00l\x00l\x00o\x00\x00\x00", "utf-16-le"),
        (14, b"test\x00", "utf-8"),
        (21, b"joe\x00", "utf-8"),
        (25, b"\x00w\x00o\x00r\x00l\x00d", "utf-16-be"),
    ]

    # Test arbitrary number of null bytes in between strings.
    data = b"\x00\x00\x00hello\x00\x00\x00\x00\x00\x00w\x00o\x00r\x00l\x00d\x00\x00\x00\x00\x00\x00"
    assert list(find_string_data(data)) == [
        (3, b"hello\x00", "utf-8"),
        (14, b"w\x00o\x00r\x00l\x00d\x00\x00\x00", "utf-16-le"),
    ]

    # Test falling back on single character strings.
    data = b"1\x000\x00\x00\x00hello\x00world\x00"
    assert list(find_string_data(data)) == [
        (0, b"1\x000\x00\x00\x00", "utf-16-le"),
        (6, b"hello\x00", "utf-8"),
        (12, b"world\x00", "utf-8"),
    ]
    data = b"1\x000\x00hello\x00world\x00"
    assert list(find_string_data(data)) == [
        (0, b"1\x00", "utf-8"),
        (2, b"0\x00", "utf-8"),
        (4, b"hello\x00", "utf-8"),
        (10, b"world\x00", "utf-8"),
    ]



"""
Tests the CLI tools.
"""

import hashlib
import json
import os
import sys

import pytest

from kordesii import cli


def test_get_malware_repo_path(tmpdir, strings_exe):
    """Tests generating malware repo path."""
    malware_repo = tmpdir.mkdir("malware_repo")
    sample_path = cli._get_malware_repo_path(str(strings_exe), str(malware_repo))
    assert sample_path == str(malware_repo / "e1b6" / "e1b6be6c0c2db8b3d4dca56062ca6301")


def test_add_to_malware_repo(tmpdir, strings_exe):
    """Tests adding a file to the malware repo."""
    malware_repo = tmpdir.mkdir("malware_repo")
    sample_path = cli._add_to_malware_repo(str(strings_exe), str(malware_repo))
    expected_sample_path = malware_repo / "e1b6" / "e1b6be6c0c2db8b3d4dca56062ca6301"
    assert sample_path == str(expected_sample_path)
    assert expected_sample_path.exists()
    assert expected_sample_path.read_binary() == strings_exe.read_binary()


def test_list(tmpdir, script_runner, Sample_decoder):
    """
    Tests displaying a list of decoders.

    (This is also where we test the decoder registration flags.)
    """
    # First ensure our Sample decoder is registered via entry_points.
    ret = script_runner.run("kordesii", "list", "--json")
    print(ret.stdout)
    print(ret.stderr, file=sys.stderr)
    assert ret.success

    results = json.loads(ret.stdout)
    assert len(results) > 1
    for name, source_name, author, description in results:
        if name == u"Sample":
            assert source_name == u"kordesii"
            assert author == u"DC3"
            assert description == u"Sample decoder"
            break
    else:
        pytest.fail("Sample decoder was not listed.")

    # Now try adding a decoder using the --decoder-dir flag.
    decoder_dir = Sample_decoder.dirname
    ret = script_runner.run("kordesii", "--decoder-dir", str(decoder_dir), "list", "--json")
    print(ret.stdout)
    print(ret.stderr, file=sys.stderr)
    assert ret.success

    results = json.loads(ret.stdout)
    assert len(results) > 1
    for name, source_name, author, description in results:
        if source_name == str(decoder_dir):
            assert name == u"Sample"
            assert author == u"DC3"
            assert description == u"Sample decoder"
            break
    else:
        pytest.fail("Sample decoder from decoder directory was not listed.")

    # If we set --decoder-source we should only get our registered decoder from the directory.
    decoder_dir = Sample_decoder.dirname
    ret = script_runner.run(
        "kordesii", "--decoder-dir", str(decoder_dir), "--decoder-source", str(decoder_dir), "list", "--json"
    )
    print(ret.stdout)
    print(ret.stderr, file=sys.stderr)
    assert ret.success

    results = json.loads(ret.stdout)
    assert results == [[u"Sample", str(decoder_dir), u"DC3", u"Sample decoder"]]


def test_parse(tmpdir, script_runner, strings_exe):
    """Test running a decoder"""
    # Change working directory so we can cleanup outputted files.
    cwd = str(tmpdir)

    # Run the foo parser on the test input file.
    ret = script_runner.run("kordesii", "parse", "Sample", str(strings_exe), "-t", "0", cwd=cwd)
    print(ret.stdout)
    print(ret.stderr, file=sys.stderr)
    assert ret.success
    assert (
        ret.stdout
        == """\
----Decoded Strings----

Hello World!
Test string with key 0x02
The quick brown fox jumps over the lazy dog.
Oak is strong and also gives shade.
Acid burns holes in wool cloth.
Cats and dogs each hate the other.
Open the crate but don't break the glass.
There the flood mark is ten inches.
1234567890
CreateProcessA
StrCat
ASP.NET
kdjsfjf0j24r0j240r2j09j222
32897412389471982470
The past will look brighter tomorrow.
Cars and busses stalled in sand drifts.
The jacket hung on the back of the wide chair.
32908741328907498134712304814879837483274809123748913251236598123056231895712

----Debug----

[+] Found XOR encrypt function at: 0x401000
[+] IDA return code = 0

"""
    )

    # Test the json formating
    ret = script_runner.run("kordesii", "parse", "--json", "Sample", str(strings_exe), "-t", "0", cwd=cwd)
    print(ret.stdout)
    print(ret.stderr, file=sys.stderr)
    assert ret.success
    assert json.loads(ret.stdout) == [
        {
            "debug": ["[+] Found XOR encrypt function at: 0x401000", "[+] IDA return code = 0"],
            "strings": [
                u"Hello World!",
                u"Test string with key 0x02",
                u"The quick brown fox jumps over the lazy dog.",
                u"Oak is strong and also gives shade.",
                u"Acid burns holes in wool cloth.",
                u"Cats and dogs each hate the other.",
                u"Open the crate but don't break the glass.",
                u"There the flood mark is ten inches.",
                u"1234567890",
                u"CreateProcessA",
                u"StrCat",
                u"ASP.NET",
                u"kdjsfjf0j24r0j240r2j09j222",
                u"32897412389471982470",
                u"The past will look brighter tomorrow.",
                u"Cars and busses stalled in sand drifts.",
                u"The jacket hung on the back of the wide chair.",
                u"32908741328907498134712304814879837483274809123748913251236598123056231895712",
            ],
        }
    ]


def test_add_testcase(tmpdir, script_runner, strings_exe):
    """Tests adding a decoder testcase."""
    # Create a dummy malware repo
    malware_repo = tmpdir.mkdir("malware_repo")

    # Create a dummy test case directory
    test_case_dir = tmpdir.mkdir("testcases")

    # Add a test case for our sample decoder.
    ret = script_runner.run(
        "kordesii",
        "test",
        "Sample",
        "--testcase-dir",
        str(test_case_dir),
        "--malware-repo",
        str(malware_repo),
        "--add",
        str(strings_exe),
    )
    print(ret.stdout)
    print(ret.stderr, file=sys.stderr)
    assert ret.success

    # Ensure test file got placed in the right location.
    test_sample = malware_repo / "e1b6" / "e1b6be6c0c2db8b3d4dca56062ca6301"
    assert test_sample.exists()
    assert test_sample.read_binary() == strings_exe.read_binary()

    # Ensure the test case was created correctly.
    test_case_file = test_case_dir / "Sample.json"
    assert test_case_file.exists()
    assert json.loads(test_case_file.read_text("utf8")) == [
        {
            u"debug": [u"[+] Found XOR encrypt function at: 0x401000", u"[+] IDA return code = 0"],
            u"input_file": os.path.join("{MALWARE_REPO}", "e1b6", "e1b6be6c0c2db8b3d4dca56062ca6301"),
            u"strings": [
                "1234567890",
                "32897412389471982470",
                "32908741328907498134712304814879837483274809123748913251236598123056231895712",
                "ASP.NET",
                "Acid burns holes in wool cloth.",
                "Cars and busses stalled in sand drifts.",
                "Cats and dogs each hate the other.",
                "CreateProcessA",
                "Hello World!",
                "Oak is strong and also gives shade.",
                "Open the crate but don't break the glass.",
                "StrCat",
                "Test string with key 0x02",
                "The jacket hung on the back of the wide chair.",
                "The past will look brighter tomorrow.",
                "The quick brown fox jumps over the lazy dog.",
                "There the flood mark is ten inches.",
                "kdjsfjf0j24r0j240r2j09j222",
            ],
        }
    ]

    # Now test the deletion of the test case.
    ret = script_runner.run(
        "kordesii",
        "test",
        "Sample",
        "--testcase-dir",
        str(test_case_dir),
        "--malware-repo",
        str(malware_repo),
        "--delete",
        str(strings_exe),
    )
    print(ret.stdout)
    print(ret.stderr, file=sys.stderr)
    assert ret.success

    # Make sure we did NOT remove the file from the malware repo.
    assert test_sample.exists()
    assert test_sample.read_binary() == strings_exe.read_binary()

    # Check that the test case has been removed, but the test case file still exists.
    assert test_case_file.exists()
    assert json.loads(test_case_file.read_text("utf8")) == []


def test_add_filelist_testcase(tmpdir, script_runner):
    """Tests bulk adding testcases with --add-filelist flag."""
    # Create a dummy malware repo
    malware_repo = tmpdir.mkdir("malware_repo")

    # Create a dummy test case directory
    test_case_dir = tmpdir.mkdir("testcases")

    # Create a file list of paths.
    filelist = []
    for i in range(10):
        file = tmpdir / "file_{}".format(i)
        data = "this is file {}".format(i).encode()
        file.write_binary(data)
        filelist.append((str(file), hashlib.md5(data).hexdigest()))

    filelist_txt = tmpdir / "filelist.txt"
    filelist_txt.write_text(u"\n".join(file_path for file_path, _ in filelist), "utf8")

    # Add a test case for our sample decoder.
    ret = script_runner.run(
        "kordesii",
        "test",
        "Sample",
        "--testcase-dir",
        str(test_case_dir),
        "--malware-repo",
        str(malware_repo),
        "--add-filelist",
        str(filelist_txt),
    )
    print(ret.stdout)
    print(ret.stderr, file=sys.stderr)
    assert ret.success

    # Ensure a sample was added for each file and exists in the testcase.
    test_case_file = test_case_dir / "Sample.json"
    assert test_case_file.exists()
    testcases = json.loads(test_case_file.read_text("utf8"))
    input_files = [testcase[u"input_file"] for testcase in testcases]
    assert len(input_files) == len(filelist)
    for _, md5 in filelist:
        test_sample = malware_repo / md5[:4] / md5
        assert test_sample.exists()
        assert hashlib.md5(test_sample.read_binary()).hexdigest() == md5
        assert os.path.join("{MALWARE_REPO}", md5[:4], md5) in input_files



"""
Tests for bug fixes reported on GitHub.
"""

import pytest


@pytest.mark.in_ida
def test_issue_7():
    """Tests the use of WIDE_STRING for read_data()"""
    from kordesii.utils import function_tracing

    emulator = function_tracing.Emulator()
    context = emulator.new_context()

    wide_string = b"/\x00f\x00a\x00v\x00.\x00i\x00c\x00o\x00"
    context.memory.write(0x123000, wide_string)
    assert context.read_data(0x123000, data_type=function_tracing.WIDE_STRING) == wide_string
    wide_string = b"\x00/\x00f\x00a\x00v\x00.\x00i\x00c\x00o"
    context.memory.write(0x123000, wide_string)
    assert context.read_data(0x123000, data_type=function_tracing.WIDE_STRING) == wide_string


@pytest.mark.in_ida_x86
@pytest.mark.in_ida_arm
def test_function_case_senstivity():
    """Tests issue with case sensitivity when hooking functions."""
    from kordesii.utils import function_tracing
    from kordesii.utils.function_tracing.call_hooks import stdlib

    emulator = function_tracing.Emulator()

    # Test with known builtin func
    assert emulator.get_call_hook("lstrcpya") == stdlib.builtin_funcs.strcpy
    assert emulator.get_call_hook("lStrcpyA") == stdlib.builtin_funcs.strcpy
    assert emulator.get_call_hook("lstrcpyA") == stdlib.builtin_funcs.strcpy

    # Test user defined
    def dummy(ctx, func_name, func_args):
        return

    assert emulator.get_call_hook("SuperFunc") is None
    assert emulator.get_call_hook("SUPERfunc") is None
    assert emulator.get_call_hook("superfunc") is None
    emulator.hook_call("SuperFunc", dummy)
    assert emulator.get_call_hook("SuperFunc") == dummy
    assert emulator.get_call_hook("SUPERfunc") == dummy
    assert emulator.get_call_hook("superfunc") == dummy
